////////////////////////////////////////////////////////////////////////////////
//Description:  Second timer                                                   /
//Firmware:     0178-00-02                                                     /
//Hardware:     None                                                           /
//Processor:    PIC12F508                                                      /
////////////////////////////////////////////////////////////////////////////////
//1 sec/1 min/1 hour clock generator from 32768 crystal.                       /
//                                                                             /
//Pin   Function                                                               /
// 1    Vdd                                                                    /
// 2    XTAL1                                                                  /
// 3    XTAL2                                                                  /
// 4    Low on pin holds the counters in reset                                 /
// 5    Hours output                                                           /
// 6    Minutes output                                                         /
// 7    Seconds output                                                         /
// 8    Gnd                                                                    /
////////////////////////////////////////////////////////////////////////////////
//Revision history                                                             /
//No.   Date        Description                                                /
//-----------------------------------------------------------------------------/
//00    2008/02/14  Initial release                                            /
//01    2008/02/18  Added halt mode clearing of port image                     /
//02    2008/02/29  Added correction factor                                    /
////////////////////////////////////////////////////////////////////////////////
#define     debugme             0           //0=standalone / 1=debug
////////////////////////////////////////////////////////////////////////////////
#include    "017800.h"
////////////////////////////////////////////////////////////////////////////////
#byte       timer0              =0x01
#bit        timer0msb           =timer0.7
#byte       bport               =0x06
unsigned    int32               trimcounter;
unsigned    int8                trimpolarity;
////////////////////////////////////////////////////////////////////////////////
//the use of an array and looping code in the reloader forces this constant
//to be compiled as a look up table, making it easy to find and replace for
//device-device calibration.  Set these to FF,FF,FF,FF,00 to set no calibration
unsigned    int8 const          trimconstant[5] ={0x00,0x01,0x24,0x26,0x00};
////////////////////////////////////////////////////////////////////////////////







////////////////////////////////////////////////////////////////////////////////
void reloadtrim(void){
unsigned int8 temp[5];

for (trimpolarity=0;trimpolarity<5;trimpolarity++)
    temp[trimpolarity]=trimconstant[trimpolarity];
    

trimcounter=make32(temp[0],temp[1],temp[2],temp[3]);
trimpolarity=temp[4];
}
////////////////////////////////////////////////////////////////////////////////






void main(void){
////////////////////////////////////////////////////////////////////////////////
unsigned    int1    t0bit=0;
unsigned    int8    scounter=0;
unsigned    int16   mcounter=0;
unsigned    int16   hcounter=0;
unsigned    int8    bitmap=7;
unsigned    int1    trimslow=0;
unsigned    int1    trimfast=0;
////////////////////////////////////////////////////////////////////////////////
bport=0x07;                             //all counter bits 1
set_tris_b(0xf8);                       //counter bits are outputs
////////////////////////////////////////////////////////////////////////////////
#asm
movlw   0x4F;                           //set prescaler to WDT, 1:1=18mS
option;
#endasm
////////////////////////////////////////////////////////////////////////////////

reloadtrim();

while(true){
////////////////////////////////////////////////////////////////////////////////
restart_wdt();

if (!(bport & 0x08))                    //halt mode button is down=low
    {
    timer0=0x00;                        //clear timer
    scounter=0;                         //clear counters
    mcounter=0;
    hcounter=0;
    t0bit=0;                            //clear overflow bit image to match tmr0
    bport=0x07;                         //set all the pins to logic high
    bitmap=0x07;                        //clear port image as well, all high
    }

if (t0bit)
    {
    if (!timer0msb)                     //look for low to high transitions
        {
        t0bit=0;                        //this is an overflow, 1-to-0
        //----------------------------------------------------------------------
        trimcounter--;                  //clock error adjust
        if (!trimcounter)
            {
            reloadtrim();
            if (!trimpolarity)
                trimslow=1;
            else
                trimfast=1;
            }
        //----------------------------------------------------------------------
        if (!trimfast)                  //normal/slow clock counts every overflow
            {
            scounter++;
            mcounter++;
            hcounter++;
            }
        //----------------------------------------------------------------------
        else                            //fast clock drops an overflow
            trimfast=0;
        //----------------------------------------------------------------------
        if (trimslow)                   //slow clock gets an extra overflow
            {
            trimslow=0;
            scounter++;
            mcounter++;
            hcounter++;
            }
        //----------------------------------------------------------------------
        if (scounter>=16)               //[16/32]hz = 0.5 sec edge rate
            {
            bitmap ^= 1;
            scounter-=16;
            }
        if (mcounter>=960)              //[960/32]hz = 30 sec edge rate
            {
            bitmap ^= 2;
            mcounter-=960;
            }
        if (hcounter>=57600)            //[57600/32]hz = 1800 sec edge rate
            {
            bitmap ^= 4;
            hcounter-=57600;
            }
        bport=bitmap;
        //----------------------------------------------------------------------
        }
    }
else
    {
    if (timer0msb)                      //just to keep track of toggles
        {
        t0bit=1;
        }
    }

////////////////////////////////////////////////////////////////////////////////
}}
////////////////////////////////////////////////////////////////////////////////

